@@ -3,14 +3,14 @@
 # Shake-Files
 
 Shake-Files allows your application to flexibly and efficiently handle file
-uploads. 
+uploads.
 
 You can create different sets of uploads - one for document attachments, one
-for photos, etc. - and configure them to save tthe files in different places, 
+for photos, etc. - and configure them to save tthe files in different places,
 creating the directories on demand, according to a pattern defined by you.
 
 
-Copyright © 2010-2012 by Lúcuma labs (http://lucumalabs.com).  
+Copyright © 2010-2012 by Lúcuma labs (http://lucumalabs.com).
 MIT License. (http://www.opensource.org/licenses/mit-license.php)
 
 """
@@ -28,7 +28,7 @@ from werkzeug.exceptions import RequestEntityTooLarge, UnsupportedMediaType
 
 from . import utils
 
-__version__ = '1.4.1'
+__version__ = '1.5'
 
 
 IMAGES = ('.jpg', '.jpeg', '.png', '.gif', '.bmp',)
@@ -37,11 +37,11 @@ AUDIO = ('.wav', '.mp3', '.aac', '.ogg', '.oga', '.flac',)
 VIDEO = ('.mp4', '.mpg', '.avi', '.mkv', '.flv', )
 
 DOCUMENTS = ('.pdf', '.rtf', '.txt', '.md', '.mdown', '.rst',
-    '.odf', '.odp', '.ods', '.odg', '.ott', '.otp', '.ots', '.otg',
-    '.pages', '.key', '.numbers', '.gnumeric', '.abw',
-    '.doc', '.ppt', '.xls', '.vsd',
-    '.docx', '.pptx', '.xlsx', '.vsx',
-    )
+             '.odf', '.odp', '.ods', '.odg', '.ott', '.otp', '.ots', '.otg',
+             '.pages', '.key', '.numbers', '.gnumeric', '.abw',
+             '.doc', '.ppt', '.xls', '.vsd',
+             '.docx', '.pptx', '.xlsx', '.vsx',
+             )
 
 DATA = ('.csv', '.json', '.xml', '.ini', '.plist', '.yaml', '.yml',)
 
@@ -66,10 +66,10 @@ OPENXML_MIMETYPES = {
 
 
 class FileStorage(object):
-    
+
     def __init__(self, base_path, base_url='/static/media/',
-            upload_to='{yyyy}/{mm}/{dd}/', secret=False, prefix='',
-            name=None, allowed=DEFAULT, denied=None, max_size=None):
+                 upload_to='{yyyy}/{mm}/{dd}/', secret=False, prefix='',
+                 name=None, allowed=DEFAULT, denied=None, max_size=None):
         """
         Except for `base_path`, all of these parameters are optional,
         so only bother setting the ones relevant to your application.
@@ -83,12 +83,12 @@ class FileStorage(object):
         base_url
         :   The base path used when building the file's URL. By default
             is `/static/media`.
-            
+
             MEDIA_URL = '/media'
 
         upload_to
-        :   A pattern used to build the upload path on the fly. 
-            The wildcards avaliable are: 
+        :   A pattern used to build the upload path on the fly.
+            The wildcards avaliable are:
 
                 {yyyy},{yy}: Year
                 {mm}, {m}: Month (0-padded or not)
@@ -100,7 +100,7 @@ class FileStorage(object):
                 {a+}: Filename first letters
                 {z+}: Filename last letters
                 {r+}: Random letters and/or numbers
-             
+
             Instead of a string, this can also be a callable.
 
         secret
@@ -121,7 +121,7 @@ class FileStorage(object):
             of them. If the uploaded file doesn't have one of these
             extensions, an `UnsupportedMediaType` exception will be
             raised.
-             
+
             Shake-Files come with some pre-defined extension lists:
 
             IMAGES = ('.jpg', '.jpeg', '.png', '.gif', '.bmp')
@@ -150,13 +150,13 @@ class FileStorage(object):
         :   Maximum file size, in bytes, that file can have.
             Note: The `request` attribute `max_content_length`, if defined,
             has higher priority.
-        
+
         """
         self.base_path = base_path.rstrip('/')
         self.base_url = base_url.rstrip('/')
         try:
             os.makedirs(os.path.realpath(base_path))
-        except (OSError), e:
+        except (OSError) as e:
             if e.errno != errno.EEXIST:
                 raise
 
@@ -167,23 +167,19 @@ class FileStorage(object):
         self.allowed = allowed or []
         self.denied = denied or []
         self.max_size = max_size
-    
-    def __repr__(self):
-        return '<%s "%s" secret=%s>' % (self.__class__.__name__,
-            self.upload_to, self.secret)
 
     def save(self, filesto, upload_to=None, secret=None, prefix=None,
-            name=None, allowed=None, denied=None, max_size=None):
+             name=None, allowed=None, denied=None, max_size=None):
         """
         Except for `filesto`, all of these parameters are optional, so only
         bother setting the ones relevant to *this upload*.
 
         filesto
         :   A `werkzeug.FileStorage`.
-        
+
         upload_to
-        :   A pattern used to build the upload path on the fly. 
-            The wildcards avaliable are: 
+        :   A pattern used to build the upload path on the fly.
+            The wildcards avaliable are:
 
                 {yyyy},{yy}: Year
                 {mm}, {m}: Month (0-padded or not)
@@ -195,7 +191,7 @@ class FileStorage(object):
                 {a+}: Filename first letters
                 {z+}: Filename last letters
                 {r+}: Random letters and/or numbers
-             
+
             Instead of a string, this can also be a callable.
 
         secret
@@ -216,7 +212,7 @@ class FileStorage(object):
             of them. If the uploaded file doesn't have one of these
             extensions, an `UnsupportedMediaType` exception will be
             raised.
-             
+
             Shake-Files come with some pre-defined extension lists:
 
             IMAGES = ('.jpg', '.jpeg', '.png', '.gif', '.bmp')
@@ -245,7 +241,7 @@ class FileStorage(object):
         :   Maximum file size, in bytes, that file can have.
             Note: The attribute `max_content_length` defined in the
             `request` object has higher priority.
-        
+
         """
         if not filesto:
             return None
@@ -266,19 +262,21 @@ class FileStorage(object):
         oname, ext = os.path.splitext(original_filename)
         if name:
             new_name = name(original_filename) if callable(name) else name
-        else:    
-            new_name = utils.get_random_filename() if secret else prefix + oname
+        else:
+            new_name = utils.get_random_filename(
+            ) if secret else prefix + oname
 
         filepath = utils.format_path(tmplpath, new_name)
-        
-        filename = utils.get_unique_filename(self.base_path, filepath, new_name,
+
+        filename = utils.get_unique_filename(
+            self.base_path, filepath, new_name,
             ext=ext)
         fullpath = utils.make_dirs(self.base_path, filepath)
 
         fullpath = os.path.join(fullpath, filename)
         filesto.save(fullpath)
         filesize = os.path.getsize(fullpath)
-        
+
         # Post validation
         if max_size and filesize > max_size:
             try:
@@ -286,19 +284,17 @@ class FileStorage(object):
             except:
                 pass
             raise RequestEntityTooLarge
-        
+
         _, ext = os.path.splitext(filepath)
         content_type = OPENXML_MIMETYPES.get(ext, filesto.content_type)
 
         data = {
-            'name': filename,
-            'relpath': filepath,
+            'path': filepath,
             'size': filesize,
             'content_type': content_type,
-            'remote_url': None,
         }
-        return File(data, self.base_path, self.base_url)
-    
+        return File(data)
+
     __call__ = save
 
     def validate(self, filesto, allowed=None, denied=None, max_size=None):
@@ -308,15 +304,15 @@ class FileStorage(object):
             filesto.seek(0, 2)
             content_length = filesto.tell()
             filesto.seek(0, 0)
-        
+
         if max_size and content_length > max_size:
             raise RequestEntityTooLarge
-        
+
         original_filename = filesto.filename
         name, ext = os.path.splitext(original_filename)
         ext = ext.lower()
         self.check_file_extension(ext, allowed, denied)
-    
+
     def check_file_extension(self, ext, allowed=None, denied=None):
         allowed = allowed or self.allowed
         denied = denied or self.denied
@@ -326,72 +322,73 @@ class FileStorage(object):
         if denied and ext in denied:
             raise UnsupportedMediaType()
 
+    def __repr__(self):
+        return '<FileStorage "%s" secret=%s>' % (self.upload_to, self.secret)
+
 
 class File(object):
     """File data wrapper.
     """
-    
-    def __init__(self, data, base_path, base_url):
-        data = data or {}
-        self.name = data.get('name', u'')
-        self.relpath = data.get('relpath', '').strip('/')
-        self.size = data.get('size', u'')
-        self.type = self.content_type = data.get('content_type', u'')
-        self.remote_url = data.get('remote_url')
 
+    def __init__(self, data):
+        data = data or {}
+        path = data.get('path', u'').strip('/')
+        if not path:
+            path = u'/%s/%s' % (
+                data.get('relpath', u''), data.get('name', u''))
+        else:
+            path = u'/' + path
+
+        self.path = path
+        self.size = data.get('size', u'')
+        self.content_type = data.get('content_type', u'')
+        self.url = data.get('url') or path
         self.width = data.get('width')
         self.height = data.get('height')
-        
-        self.base_path = base_path.rstrip('/')
-        self.base_url = base_url.rstrip('/')
-        
-        self.is_image = self.content_type.startswith('image/')
 
-        path = ''
-        if self.relpath:
-            path = os.path.join(self.base_path, self.relpath, self.name)
-        self.path = path
+    @property
+    def name(self):
+        return os.path.basename(self.path)
+
+    @property
+    def relpath(self):
+        return os.path.basedir(self.path)
+
+    @property
+    def is_image(self):
+        return self.content_type.startswith('image/')
 
-        url = ''
-        if self.remote_url:
-            url = self.remote_url
-        elif self.relpath:
-            url = '%s/%s/%s' % (self.base_url, self.relpath, self.name)    
-        self.url = url
-    
     @property
     def __dict__(self):
         return self.to_dict()
 
     def __nonzero__(self):
         return bool(self.path)
-    
+
     def __repr__(self):
         return repr(self.to_dict())
-    
+
     def remove(self):
         try:
             os.remove(self.path)
         except OSError:
             pass
-    
+
     def to_dict(self):
         """Serialize to a dictionary.
         """
         data = {
-            'name': self.name,
-            'relpath': self.relpath,
+            'path': self.path,
             'size': self.size,
             'content_type': self.content_type,
             'url': self.url,
-            'remote_url': self.remote_url,
         }
         if self.width:
             data['width'] = self.width
         if self.height:
             data['height'] = self.height
         return data
-    
+
     def to_json(self):
         """Serialize to JSON.
         """
@@ -400,37 +397,32 @@ class File(object):
 
 if sqlalchemy_available:
     class FileType(TypeDecorator):
+
         """Saves a File as a JSON-encoded string and reads it back as
         a File."""
 
         impl = Text
         filedata_class = File
 
-        def __init__(self, base_path, base_url, default=None, **kwargs):
-            """
-            """
-            self.base_path = base_path
-            self.base_url = base_url
+        def __init__(self, default=None, **kwargs):
             self.default = default
             self.kwargs = kwargs
             TypeDecorator.__init__(self, **kwargs)
-        
+
         def process_bind_param(self, value, dialect):
             if value is None:
                 return None
             if hasattr(value, 'to_json'):
                 return value.to_json()
             return utils.json.dumps(value)
-        
+
         def process_result_value(self, value, dialect):
             if not value:
                 if self.default:
                     return self.default
                 return None
             data = utils.json.loads(value)
-            return self.filedata_class(data, self.base_path, self.base_url)
-        
-        def copy(self):
-            return self.__class__(self.base_path, self.base_url,
-                default=self.default, **self.kwargs)
+            return self.filedata_class(data)
 
+        def copy(self):
+            return self.__class__(default=self.default, **self.kwargs)
